#include <TreqThread.h>

///////////////////////////////////////////////////////////////////////////////
/**Constructor.
*/
TIPbusError::TIPbusError(QString Mess)
{
Message=Mess;
fatalError=false;
errorProcessed=0;
}
//////////////////////////////////////////////////////////////////////////
/**Constructor.*/
TreqThread::TreqThread(QObject *parent): QObject(parent)
{
networkMTU=1500; //TODO ability to change MTU size
TC_socket=new QUdpSocket(0);
packetID=0;
connect(TC_socket,SIGNAL(readyRead()),this,SLOT(getPacket()));
connect(TC_socket,SIGNAL(readyRead()),this,SLOT(processRequest()));

connect(TC_socket, SIGNAL(connected()),this, SLOT(ConnectMessage()));
connect(TC_socket, SIGNAL(disconnected()),this, SLOT(ByeMessage()));
connect(TC_socket,SIGNAL(stateChanged(QAbstractSocket::SocketState)),this,SLOT(printState(QAbstractSocket::SocketState)));

status_header=(0xF1<<24) | 0x20;
firstStatusPacket=true;
requestInProcess=false;
timeoutStatusRequest=false;
timeoutStatusGearBox=0;
inResponseRecovery=false;
inRequestRecovery=false;
errorProcessing=0;
reissueCount=0;
}
//////////////////////////////////////////////////////////////////////////
TreqThread::~TreqThread()
{
if (!firstStatusPacket)
	{

	for (uint i=0;i<nResponseBuffers;i++)
		{
		delete IPbusTransactions[i];
		}
	delete [] IPbusTransactions;
	}
}
//////////////////////////////////////////////////////////////////////////
void TreqThread::connectClient(QString addStr_in, int port_in)
{
//clear queue to allow reconnect
availTxTrans.clear();
//reqQueue.clear();
//recvTrans.clear();

addStr=addStr_in;
port=port_in;
hostAdd.setAddress(addStr);
cout << "set IP" << hostAdd.toString().toStdString() << endl;
if (TC_socket->state ()==QAbstractSocket::ConnectedState)
	TC_socket->disconnectFromHost();

firstStatusPacket=true;
TC_socket->connectToHost(addStr,port,QIODevice::ReadWrite);

//this mutex is previously locked by connect request
//dispatchMutex.unlock();
}
//////////////////////////////////////////////////////////////////////////
void TreqThread::addControlRequest(TreqParam reqParam)
{
	//cout << "enqueue request" << endl;
reqQueue.enqueue(reqParam);
}
//////////////////////////////////////////////////////////////////////////
int TreqThread::getCurTransBuff()
{
int curBuf;

curBuf=availTxTrans.head();

return curBuf;
}
//////////////////////////////////////////////////////////////////////////
/**Method managing error.
*/
void TreqThread::manageTimeoutError()
{
QString tmr_mess=QString("timeout error at %1").arg(QDateTime::currentDateTime().toString("hh:mm:ss.zzz"));
cout << tmr_mess.toStdString() << endl;

;
//confirm that all packet are not received
if (recvTrans.isEmpty())
	{
	cout << "manageTimeoutError(): no problem: ";
	cout<< dec << "recvTrans.isEmpty()=" << recvTrans.isEmpty() << ", reqQueue.isEmpty()=" <<reqQueue.isEmpty();
	cout<< dec << ", requestInProcess=" << requestInProcess	<< endl;
	//emit rearmTimeoutTimer();
	//if (!requestInProcess && reqQueue.isEmpty())
		//dispatchMutex.unlock();

	return;
	}

if (timeoutStatusRequest)
	{
	timeoutStatusGearBox++;
	if (timeoutStatusGearBox>10)
		{
		QString mess=QString(tr("IPBUS/TreqTthread: Error occured, status request failed. Totally lost"));
		cout << mess.toStdString() << endl;
		TIPbusError IPbusTransactions(mess);
		IPbusTransactions.fatalError=true;
		dispatchMutex.unlock();
		throw (mess);
		}
	}

timeoutStatusRequest=true;
requestStatusPacket();
emit rearmTimeoutTimer();

}
//////////////////////////////////////////////////////////////////////////
/**Error recovery.
*/
void TreqThread::errorRecovery()
{
qint32 nextExpectedPacketID;
qint32 searchedPacketID;

if (recvTrans.isEmpty())
	{
	cout << "ErrorRecovery(): No problem" << endl;
	inResponseRecovery=false;
	inRequestRecovery=false;
	return;
	}
if (errorProcessing>2*statusPacket.nResponseBuffers)
	{
	QString mess=QString(tr("IPBUS/TreqTthread: More errors than available buffer, totally lost"));
	TIPbusError IPbusTransactions(mess);
	IPbusTransactions.fatalError=true;
	IPbusTransactions.errorProcessed=errorProcessing;
	cout << mess.toStdString() << endl;
	dispatchMutex.unlock();
	throw (mess);
	}
errorProcessing++;

//from the status determine how many (and which) paquets were lost from hardware to client
// => resent request
// or 
//how many control paquets were lost (from client to hardware) 
// => Full (and all) control requests need to be resent

//for (int i=0;i<recvTrans.size();i++)
for (int i=0;i<1;i++) //1 a la fois pour debug
	{
	searchedPacketID=IPbusTransactions[recvTrans.at(i)]->expectedPacketID();
	nextExpectedPacketID=_IPBusPacketID(statusPacket.nextExpectedHeader);
	qint32 lastReqReceived;
	qint32 reqDiff;
	if (nextExpectedPacketID==1)
		lastReqReceived=0xFFFF;
	else
		lastReqReceived=nextExpectedPacketID-1;

	reqDiff=searchedPacketID-lastReqReceived;
	if (reqDiff>signed(statusPacket.nResponseBuffers))
		reqDiff=reqDiff-0xFFFF;
	if (reqDiff<-1*signed(statusPacket.nResponseBuffers))
		reqDiff=reqDiff+0xFFFF;

	cout << "searching IPBUS packet 0x" << hex << IPbusTransactions[recvTrans.at(i)]->expectedPacketID() << ", currently target expecting 0x" << nextExpectedPacketID <<", reqDiff=" << dec << reqDiff << endl;
	
	// check zero crossing (ex nextExpected 0x0001, missing response 0xFFFF). Unsigned subtraction
	if (reqDiff>0)		
		{//zero not crossed
		cout << "request was lost" << endl;
		sendRequest(recvTrans.at(i),true);
		//client expected paquet is lower from the missing one (=> request lost)
		}
	else
		if (reqDiff<=0)
			{
			cout << "lastRequestReceived=0x" << hex << lastReqReceived << ", searchedPacketID=0x" << searchedPacketID << endl;
			resendPacket(searchedPacketID);
			}
	}

//rearm timeout
emit rearmTimeoutTimer();
}
//////////////////////////////////////////////////////////////////////////
/**Method implementing the background task.
Runs until either there is a communication error or a stop is requested
*/
void TreqThread::processRequest()
{
	//cout << "process request" << endl;
bool slotStillAvailable;
if (!reqQueue.isEmpty() && !requestInProcess)
	{
	//get new request	
	curParam=reqQueue.dequeue();
	requestInProcess=true;

	}

while (requestInProcess && availTxTrans.size()>0 && !inResponseRecovery && !inRequestRecovery)
	{
	slotStillAvailable=IPbusTransactions[getCurTransBuff()]->appendToTransaction(packetID,curParam);

	if (!slotStillAvailable)
		{
		//xmit previous transaction buffer
		//cout << hex << "pack=0x" << packetID <<endl;
		sendRequest(getCurTransBuff());

		availTxTrans.dequeue();
		//cout << dec << "slot available=" << availTxTrans.size() << endl;
		if (packetID==0xFFFF) 
			packetID=1;
		else
			packetID++;
		//if (curParam.type!=_done)
			//{
			////cout << "pack++" << endl;
			//IPbusTransactions[getCurTransBuff()]->appendToTransaction(packetID,curParam);
			//}
		}
	if (curParam.type==_done)
		{
			//cout << "requestInProcess=false" << endl;
		requestInProcess=false;
		}
	}
}
//////////////////////////////////////////////////////////////////////////
/** Used to release TXslots.
*/
void TreqThread::releaseTxSlot(int slot)
{
IPbusTransactions[slot]->clean();
availTxTrans.enqueue(slot);
//cout << dec << "release slot available " << availTxTrans.size() << endl;
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::ByeMessage()
{
QString str=QString(">goodbye");
emit Message(str);
//emit connected(false);
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::requestStatusPacket()
{
cout << "request status packet" << endl;
//this request is big endian always
qint32 word[16];
word[0]=status_header;
for (int i=1;i<16;i++)
	word[i]=0;
TC_socket->write((char*)word,sizeof(word));
//TC_socket->writeDatagram((char*)word,sizeof(word),*hostAdd,port);
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::resendPacket(quint16 packID)
{
cout << "resend packet ID=0x" << hex  <<packID << endl;
inResponseRecovery=true;
//this request is big endian always
quint32 request=(0x20<<24) | (packID<<8) | 0xF2;
quint32 endianCorrectedRequest;

#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
	endianCorrectedRequest=(quint32) qToBigEndian<quint32>(request);
#endif
TC_socket->write((char*)&endianCorrectedRequest,sizeof(quint32));
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::processInitialStatusPacket()
{
int IPBusmaxPacketSize;
firstStatusPacket=false;
//Size is MTU minus IP header and minus UDP header
if (statusPacket.MTU>networkMTU)
	IPBusmaxPacketSize=networkMTU-20-8;
else
	IPBusmaxPacketSize=statusPacket.MTU-20-8;
cout << dec << "MTU firmware=" << statusPacket.MTU << ", Nbuffers=" <<statusPacket.nResponseBuffers << endl;

packetID=(statusPacket.nextExpectedHeader>>8) & 0xFFFF;
//nResponseBuffers=statusPacket.nResponseBuffers;	
nResponseBuffers=statusPacket.nResponseBuffers-1;	

IPbusTransactions=new TIPbusTransaction*[nResponseBuffers];

for (uint i=0;i<nResponseBuffers;i++)
	{
	IPbusTransactions[i]=new TIPbusTransaction(IPBusmaxPacketSize/4);
	IPbusTransactions[i]->clean();
	}


for (uint i=0;i<nResponseBuffers;i++)
	availTxTrans.enqueue(i);

//first packet processed, ready to go!!
//cout << "1st status received" << endl;
connectMutex.unlock();
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::getPacket()
{
uint char_cnt;
//char_cnt= TC_socket->bytesAvailable();
char_cnt= TC_socket->pendingDatagramSize();
//quint32 header;
//cout << dec << "get packet " << char_cnt << endl;
//cout << "packet size=" << sizeof(TstatusPacket) << endl;

//while(char_cnt>=64)
while(TC_socket->hasPendingDatagrams())
	{
	char_cnt= TC_socket->pendingDatagramSize();
	if (char_cnt%4)
		{
		QString mess=QString(tr("IPBUS/TreqTthread: Data transmission error (getPacket), received %1 bytes, not a multiple of 4"))
					.arg(char_cnt);
		TIPbusError IPbusTransactions(mess);
		throw (mess);
		break;
		}

	receivedWords=char_cnt/4;
	//cout << dec << "char_cnt=" << char_cnt << endl;
	TC_socket->readDatagram((char*)datagram,char_cnt,&hostAdd,&port);
    //cout << "peek char_cnt " << char_cnt << endl;
	//if (header==status_header) //status packet (big endian always)
	//for (int i=0;i<receivedWords;i++)
	//for (int i=receivedWords-4;i<receivedWords;i++)
	//cout << dec << "rec i=" << i << " " << hex << datagram[i] << endl;
	if (datagram[0]==status_header) //status packet (big endian always)
		{
		if (char_cnt>=sizeof(TstatusPacket))
			{
			quint32 *temp_buf=new quint32[sizeof(TstatusPacket)/4];
			memcpy(/*dest*/temp_buf,/*src*/datagram,/*size*/sizeof(TstatusPacket));

			//qint64 read;
			//read=TC_socket->read((char*)temp_buf,sizeof(TstatusPacket)); //get status
			//cout <<dec << "readhere=" << read << endl;
			//for (int i=0;i<16;i++)
				//cout << dec << "i=" << i << hex << " 0x" << (temp_buf[i]) << endl;
			#if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
				for (int i=0;i<16;i++)
					{
					temp_buf[i]=(quint32) qFromBigEndian<quint32>(temp_buf[i]);
					if (i>=8) //correct endianness if required word by word
					   	if ((temp_buf[i]&0xF0000000)==0xF0000000)
							temp_buf[i]=(quint32) qFromBigEndian<quint32>(temp_buf[i]);
					}
			#endif
			cout << "status" << endl;
			timeoutStatusRequest=false; //requested status received
			timeoutStatusGearBox=0;
			for (int i=0;i<16;i++)
				cout << dec << "i=" << i << hex << " 0x" << temp_buf[i] << endl;
			memcpy(/*dest*/&statusPacket,/*src*/temp_buf,/*size*/sizeof(TstatusPacket));
			delete [] temp_buf;
			if (firstStatusPacket)
				processInitialStatusPacket();
			else
				errorRecovery();
			}
		}
	else
		{
		// arriving here, means that all requests were received correctly, but one paquet from
		// hardware to PC was eventually lost, all subsequent paquets are therefore OK.
		// All the valid packets are taken, and for the missing packet we wait the tiemout to kick on
		// and recover this packet
		
		//cout << dec << "rcvtrans=" << recvTrans.head() << endl;
		//cout << hex << "expected packetID=0x"<< IPbusTransactions[recvTrans.head()]->expectedPacketID();
		//cout << ", rcvPacketID=0x" <<  _IPBusPacketID(datagram[0])<< endl;

		//search corresponding packet
		
		emit rearmTimeoutTimer(); //rearm timer for each good reception
		for (int i=0;i<recvTrans.size();i++)
			{
			if (_IPBusPacketID(datagram[0])==IPbusTransactions[recvTrans.at(i)]->expectedPacketID())
				{
				parseResponse(/*slot*/recvTrans.at(i));
				if (i==0)
					inResponseRecovery=false; // packet should arrive in the same order as emitted
				else
					inResponseRecovery=true; //likely that one datagram was lost from target to PC

				releaseTxSlot(recvTrans.at(i));
				recvTrans.removeAt(i);
				break;
				}

			}
		}
	}
if (recvTrans.isEmpty())
	{
	inRequestRecovery=false; //lost requests found, can accept new request
	inResponseRecovery=false; //all response receivedn, can accept new requets
	errorProcessing=0;
	}

if (recvTrans.isEmpty() && reqQueue.isEmpty())
	{
		//cout << "unlock()" << endl;
		//errorProcessing=0;
	timeoutStatusRequest=false;
	timeoutStatusGearBox=0;
	emit stopTimeoutTimer();
	dispatchMutex.unlock();
	}

//cout <<"leave packet" << endl;
//cout << endl;
//cout << endl;
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::parseResponse(int slot)
{
	//cout << "parse" << endl;
int idx=1;
quint32 header;
for (uint trans=0;trans<IPbusTransactions[slot]->getTransactionCount();trans++) //get transaction one by one
	{
	if (idx>=receivedWords)
		{
		QString mess=QString(tr("IPBUS/TreqTthread: Missing transaction in packet 0x%1, expecting %2, received %3 transactions!"))
					.arg(IPbusTransactions[slot]->getTransactionCount(),/*fielwidth*/0,/*base*/16)
					.arg(IPbusTransactions[slot]->getTransactionCount())
					.arg(trans);
		TIPbusError IPbusError(mess);
		throw (IPbusError);
		break;
		}
	header=datagram[idx];
	idx++;
	if (_TransID(header)!=trans)
		{
		QString mess=QString(tr("IPBUS/TreqTthread: Bad transaction header in packet 0x%1, transaction number:received=%2, expected=%3."))
								.arg(IPbusTransactions[slot]->getTransactionCount(),/*fielwidth*/0,/*base*/16)
								.arg(_TransID(header))
								.arg(trans);
		//for debug, packet request dump in console
		cout << hex << "rcv packetID=0x" << IPbusTransactions[slot]->expectedPacketID();
		cout << dec << ", frameSize (in bytes)=" << IPbusTransactions[slot]->getFrameSize();
		cout << dec << ", transaction count=" << IPbusTransactions[slot]->getTransactionCount();
		cout<< endl; 

		quint32 *DATA_WORD_PTR=IPbusTransactions[slot]->dataFrame_ptr();
		for (uint word=0;word<IPbusTransactions[slot]->getFrameSize()/4;word++)
			{
			cout << dec << "req i=" << word <<" 0x" << hex << DATA_WORD_PTR[word]<< endl; 
			}

		//dump corresponding datagram received
		for (uint word=0;word<400;word++)
			{
			cout << dec << "datagram i=" << word <<" 0x" << hex << datagram[word]<< endl; 
			}


		TIPbusError IPbusError(mess);
		throw (IPbusError);
		throw (mess);
		}

	if (_TransInfoCode(header)!=0x00)
		{
		QString mess;
		mess=QString(tr("IPBUS/TreqTthread: Bad transaction header in packet 0x%1, wrong infoCode, received=%2 (should be 0x00).\nError is: "))
				.arg(IPbusTransactions[slot]->getTransactionCount(),/*fielwidth*/0,/*base*/16)
				.arg(_TransInfoCode(header));

		switch(_TransInfoCode(header))
			{
			case 0x1:
				mess+=QString(tr("bad header"));
				break;
			case 0x4 :
				mess+=QString(tr("Bus error on read"));
				break;
			case 0x5 :
				mess+=QString(tr("Bus error on write"));
				break;
			case 0x6 :
				mess+=QString(tr("Bus timout on read"));
				break;
			case 0x7 :
				mess+=QString(tr("Bus timout on write"));
				break;
			case 0xF :
				mess+=QString(tr("outbound request"));
				break;
			default:
				break;
			}
		
		//for debug, packet request dump in console
		cout << hex << "rcv packetID=0x" << IPbusTransactions[slot]->expectedPacketID();
		cout << dec << ", frameSize (in bytes)=" << IPbusTransactions[slot]->getFrameSize();
		cout << dec << ", transaction count=" << IPbusTransactions[slot]->getTransactionCount();
		cout<< endl; 

		quint32 *DATA_WORD_PTR=IPbusTransactions[slot]->dataFrame_ptr();
		for (uint word=0;word<IPbusTransactions[slot]->getFrameSize()/4;word++)
			{
			cout << dec << "req i=" << word <<" 0x" << hex << DATA_WORD_PTR[word]<< endl; 
			}

		//dump corresponding datagram received
		for (uint word=0;word<400;word++)
			{
			cout << dec << "datagram i=" << word <<" 0x" << hex << datagram[word]<< endl; 
			}
		TIPbusError IPbusError(mess);
		throw (IPbusError);
		}

	switch (_TransTypeID(header))
		{
		case 0 : //read transaction standard
		case 2 : //read transaction non incrementing
		case 4 : //read/modify/write bit
		case 5 : //read/modify/write sum
			//TC_socket->read((char*)IPbusTransactions[slot]->transactionPtr_ptr(trans),transactionHeader.field.words*sizeof(quint32));
			//cout << "transaction" << endl;
			memcpy(/*dest*/(char*)IPbusTransactions[slot]->transactionPtr_ptr(trans),/*src*/&datagram[idx],/*size*/_TransWords(header)*sizeof(quint32));
			//for (uint i=0;i<_TransWords(header);i++)
			//cout << dec << "i=" << i << hex << " 0x" << (datagram[idx+i]) << endl;
			idx+=_TransWords(header);
			break;

		case 1 : //reply to write transaction standard
		case 3 : //reply to write transaction non incrementing
			//TC_socket->write((char*)IPbusTransactions[slot]->transactionPtr_ptr(trans),transactionHeader.field.words*sizeof(quint32));
			//memcpy(/*dest*/datagram+idx,/*src*/(char*)IPbusTransactions[slot]->transactionPtr_ptr(trans),/*size*/transactionHeader.field.words*sizeof(quint32));
			//memcpy(/*dest*/(char*)IPbusTransactions[slot]->transactionPtr_ptr(trans),/*src*/&datagram[idx],/*size*/_TransWords(header)*sizeof(quint32));
			//idx+=_TransWords(header);
			break;

		default:
			QString mess=QString(tr("IPBUS/TreqTthread: Bad transaction header, wrong typeID, in transaction %1 received=%2"))
						.arg(_TransID(header))
						.arg(_TransTypeID(header));
			TIPbusError IPbusTransactions(mess);
			throw (mess);
			break;
		}
	}
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::ConnectMessage()
{
QString str=QString(">Socket connected, IP=%1 port=%2").arg(TC_socket->peerAddress().toString()).arg(TC_socket->peerPort());
emit Message(str);
//emit connected(true);
requestStatusPacket();
}
/////////////////////////////////////////////////////////////////////
void TreqThread::printState(QAbstractSocket::SocketState val)
{
cout << "socket state=" << val << endl;
}
///////////////////////////////////////////////////////////////////////////////
void TreqThread::sendRequest(int transToXmit,bool isReissue)
{
	//cout << dec << "send request, length=" << IPbusTransactions[transToXmit]->getFrameSize() << endl;
//for (uint i=0;i<IPbusTransactions[transToXmit]->getFrameSize()/4;i++)
	//cout << dec << "i=" << i << hex << " 0x" << (IPbusTransactions[transToXmit]->dataFrame_ptr()[i]) << endl;

	//TC_socket->writeDatagram((char*)IPbusTransactions[transToXmit]->dataFrame_ptr(),IPbusTransactions[transToXmit]->getFrameSize(),*hostAdd,port);
TC_socket->write((char*)IPbusTransactions[transToXmit]->dataFrame_ptr(),IPbusTransactions[transToXmit]->getFrameSize());
if (!isReissue)
	{
	recvTrans.enqueue(transToXmit); //enqueue only new request, not lost requests
	reissueCount=0;
	}
else 
	{
	cout << "isReissue (a request was lost), request again packet 0x" << hex << IPbusTransactions[transToXmit]->expectedPacketID() << endl;
	reissueCount++;
	inRequestRecovery=true;
	if (reissueCount>statusPacket.nResponseBuffers)
		{
		QString mess=QString(tr("IPBUS/TreqTthread: Error occured, too many reIssues. Totaly lost"));
		cout << mess.toStdString() << endl;
		TIPbusError IPbusTransactions(mess);
		IPbusTransactions.fatalError=true;
		dispatchMutex.unlock();
		throw (mess);
		}
	}

emit rearmTimeoutTimer();
}
