-------------------------------------------------------------------------------
-- once computation enabled it goes until end
-- CRC 32 first serial is LSB
-- data width 8
-- polynomial 32 bit :
-- x32 + x26 + x23 + x22 + x16 + x12 + x11 + x10 + x8 + x7 + x5 + x4 + x2 + x1 + 1
-- see http://www.electronicdesignworks.com/utilities/crc_generator/crc_generator.htm
-- see reference for http://www.lammertbies.nl/comm/info/crc-calculation.html
-- For 1st input DA: 1st CRC =B404D447
-- O. Bourrion 12/2013
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity crc_gen is
generic (is_check : boolean :=false);
port(
	rstn : in std_logic;
	clk : in std_logic;
	clear_crc : in std_logic;

	data_en : in std_logic;
	data_in : in std_logic_vector(7 downto 0);

	crc_is_good : out std_logic;
	checksum: out std_logic_vector(7 downto 0)
);
end crc_gen;


architecture RTL of crc_gen is
signal crc_word : std_logic_vector(31 downto 0);
-- signal crc_word_bis : std_logic_vector(31 downto 0);
-- signal crc_word_ter : std_logic_vector(31 downto 0);
signal lfsr_q: std_logic_vector (31 downto 0);
signal data_en_1ck : std_logic;
begin
-------------------------------------------------------------------------------
-- CRC computation
-------------------------------------------------------------------------------
process(rstn,clk)
begin
	if rstn='0' then
		lfsr_q<=(others=>'1');

	elsif rising_edge(clk) then
		if (clear_crc = '1') then
			lfsr_q<=(others=>'1');
		elsif (data_en = '1') then
-- LSB first
lfsr_q(0) <= data_in(7) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(1) <= data_in(7) XOR data_in(6) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(2) <= data_in(7) XOR data_in(6) XOR data_in(5) XOR lfsr_q(26) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(3) <= data_in(6) XOR data_in(5) XOR data_in(4) XOR lfsr_q(27) XOR lfsr_q(26) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31); 
lfsr_q(4) <= data_in(7) XOR data_in(5) XOR data_in(4) XOR data_in(3) XOR lfsr_q(28) XOR lfsr_q(27) XOR lfsr_q(26) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(5) <= data_in(7) XOR data_in(6) XOR data_in(4) XOR data_in(3) XOR data_in(2) XOR lfsr_q(29) XOR lfsr_q(28) XOR lfsr_q(27) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(6) <= data_in(6) XOR data_in(5) XOR data_in(3) XOR data_in(2) XOR data_in(1) XOR lfsr_q(30) XOR lfsr_q(29) XOR lfsr_q(28) XOR lfsr_q(26) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31); 
lfsr_q(7) <= data_in(7) XOR data_in(5) XOR data_in(4) XOR data_in(2) XOR data_in(0) XOR lfsr_q(31) XOR lfsr_q(29) XOR lfsr_q(27) XOR lfsr_q(26) XOR lfsr_q(24); 
lfsr_q(8) <= data_in(7) XOR data_in(6) XOR data_in(4) XOR data_in(3) XOR lfsr_q(0) XOR lfsr_q(28) XOR lfsr_q(27) XOR lfsr_q(25) XOR lfsr_q(24); 
lfsr_q(9) <= data_in(6) XOR data_in(5) XOR data_in(3) XOR data_in(2) XOR lfsr_q(1) XOR lfsr_q(29) XOR lfsr_q(28) XOR lfsr_q(26) XOR lfsr_q(25); 
lfsr_q(10) <= data_in(7) XOR data_in(5) XOR data_in(4) XOR data_in(2) XOR lfsr_q(2) XOR lfsr_q(29) XOR lfsr_q(27) XOR lfsr_q(26) XOR lfsr_q(24); 
lfsr_q(11) <= data_in(7) XOR data_in(6) XOR data_in(4) XOR data_in(3) XOR lfsr_q(3) XOR lfsr_q(28) XOR lfsr_q(27) XOR lfsr_q(25) XOR lfsr_q(24); 
lfsr_q(12) <= data_in(7) XOR data_in(6) XOR data_in(5) XOR data_in(3) XOR data_in(2) XOR lfsr_q(4) XOR lfsr_q(29) XOR lfsr_q(28) XOR lfsr_q(26) XOR lfsr_q(25) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(13) <= data_in(6) XOR data_in(5) XOR data_in(4) XOR data_in(2) XOR data_in(1) XOR lfsr_q(5) XOR lfsr_q(30) XOR lfsr_q(29) XOR lfsr_q(27) XOR lfsr_q(26) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31); 
lfsr_q(14) <= data_in(5) XOR data_in(4) XOR data_in(3) XOR data_in(1) XOR data_in(0) XOR lfsr_q(6) XOR lfsr_q(31) XOR lfsr_q(30) XOR lfsr_q(28) XOR lfsr_q(27) XOR lfsr_q(26); 
lfsr_q(15) <= data_in(4) XOR data_in(3) XOR data_in(2) XOR data_in(0) XOR lfsr_q(7) XOR lfsr_q(31) XOR lfsr_q(29) XOR lfsr_q(28) XOR lfsr_q(27); 
lfsr_q(16) <= data_in(7) XOR data_in(3) XOR data_in(2) XOR lfsr_q(8) XOR lfsr_q(29) XOR lfsr_q(28) XOR lfsr_q(24); 
lfsr_q(17) <= data_in(6) XOR data_in(2) XOR data_in(1) XOR lfsr_q(9) XOR lfsr_q(30) XOR lfsr_q(29) XOR lfsr_q(25); 
lfsr_q(18) <= data_in(5) XOR data_in(1) XOR data_in(0) XOR lfsr_q(10) XOR lfsr_q(31) XOR lfsr_q(30) XOR lfsr_q(26); 
lfsr_q(19) <= data_in(4) XOR data_in(0) XOR lfsr_q(11) XOR lfsr_q(31) XOR lfsr_q(27); 
lfsr_q(20) <= data_in(3) XOR lfsr_q(12) XOR lfsr_q(28); 
lfsr_q(21) <= data_in(2) XOR lfsr_q(13) XOR lfsr_q(29); 
lfsr_q(22) <= data_in(7) XOR lfsr_q(14) XOR lfsr_q(24); 
lfsr_q(23) <= data_in(7) XOR data_in(6) XOR lfsr_q(15) XOR lfsr_q(25) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(24) <= data_in(6) XOR data_in(5) XOR lfsr_q(16) XOR lfsr_q(26) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31); 
lfsr_q(25) <= data_in(5) XOR data_in(4) XOR lfsr_q(17) XOR lfsr_q(27) XOR lfsr_q(26); 
lfsr_q(26) <= data_in(7) XOR data_in(4) XOR data_in(3) XOR lfsr_q(18) XOR lfsr_q(28) XOR lfsr_q(27) XOR data_in(1) XOR lfsr_q(24) XOR lfsr_q(30); 
lfsr_q(27) <= data_in(6) XOR data_in(3) XOR data_in(2) XOR lfsr_q(19) XOR lfsr_q(29) XOR lfsr_q(28) XOR data_in(0) XOR lfsr_q(25) XOR lfsr_q(31); 
lfsr_q(28) <= data_in(5) XOR data_in(2) XOR data_in(1) XOR lfsr_q(20) XOR lfsr_q(30) XOR lfsr_q(29) XOR lfsr_q(26); 
lfsr_q(29) <= data_in(4) XOR data_in(1) XOR data_in(0) XOR lfsr_q(21) XOR lfsr_q(31) XOR lfsr_q(30) XOR lfsr_q(27); 
lfsr_q(30) <= data_in(3) XOR data_in(0) XOR lfsr_q(22) XOR lfsr_q(31) XOR lfsr_q(28); 
lfsr_q(31) <= data_in(2) XOR lfsr_q(23) XOR lfsr_q(29); 

-- MSB first
-- 			lfsr_q(0) <= lfsr_q(24) xor lfsr_q(30) xor data_in(0) xor data_in(6);
-- 			lfsr_q(1) <= lfsr_q(24) xor lfsr_q(25) xor lfsr_q(30) xor lfsr_q(31) xor data_in(0) xor data_in(1) xor data_in(6) xor data_in(7);
-- 			lfsr_q(2) <= lfsr_q(24) xor lfsr_q(25) xor lfsr_q(26) xor lfsr_q(30) xor lfsr_q(31) xor data_in(0) xor data_in(1) xor data_in(2) xor data_in(6) xor data_in(7);
-- 			lfsr_q(3) <= lfsr_q(25) xor lfsr_q(26) xor lfsr_q(27) xor lfsr_q(31) xor data_in(1) xor data_in(2) xor data_in(3) xor data_in(7);
-- 			lfsr_q(4) <= lfsr_q(24) xor lfsr_q(26) xor lfsr_q(27) xor lfsr_q(28) xor lfsr_q(30) xor data_in(0) xor data_in(2) xor data_in(3) xor data_in(4) xor data_in(6);
-- 			lfsr_q(5) <= lfsr_q(24) xor lfsr_q(25) xor lfsr_q(27) xor lfsr_q(28) xor lfsr_q(29) xor lfsr_q(30) xor lfsr_q(31) xor data_in(0) xor data_in(1) xor data_in(3) xor data_in(4) xor data_in(5) xor data_in(6) xor data_in(7);
-- 			lfsr_q(6) <= lfsr_q(25) xor lfsr_q(26) xor lfsr_q(28) xor lfsr_q(29) xor lfsr_q(30) xor lfsr_q(31) xor data_in(1) xor data_in(2) xor data_in(4) xor data_in(5) xor data_in(6) xor data_in(7);
-- 			lfsr_q(7) <= lfsr_q(24) xor lfsr_q(26) xor lfsr_q(27) xor lfsr_q(29) xor lfsr_q(31) xor data_in(0) xor data_in(2) xor data_in(3) xor data_in(5) xor data_in(7);
-- 			lfsr_q(8) <= lfsr_q(0) xor lfsr_q(24) xor lfsr_q(25) xor lfsr_q(27) xor lfsr_q(28) xor data_in(0) xor data_in(1) xor data_in(3) xor data_in(4);
-- 			lfsr_q(9) <= lfsr_q(1) xor lfsr_q(25) xor lfsr_q(26) xor lfsr_q(28) xor lfsr_q(29) xor data_in(1) xor data_in(2) xor data_in(4) xor data_in(5);
-- 			lfsr_q(10) <= lfsr_q(2) xor lfsr_q(24) xor lfsr_q(26) xor lfsr_q(27) xor lfsr_q(29) xor data_in(0) xor data_in(2) xor data_in(3) xor data_in(5);
-- 			lfsr_q(11) <= lfsr_q(3) xor lfsr_q(24) xor lfsr_q(25) xor lfsr_q(27) xor lfsr_q(28) xor data_in(0) xor data_in(1) xor data_in(3) xor data_in(4);
-- 			lfsr_q(12) <= lfsr_q(4) xor lfsr_q(24) xor lfsr_q(25) xor lfsr_q(26) xor lfsr_q(28) xor lfsr_q(29) xor lfsr_q(30) xor data_in(0) xor data_in(1) xor data_in(2) xor data_in(4) xor data_in(5) xor data_in(6);
-- 			lfsr_q(13) <= lfsr_q(5) xor lfsr_q(25) xor lfsr_q(26) xor lfsr_q(27) xor lfsr_q(29) xor lfsr_q(30) xor lfsr_q(31) xor data_in(1) xor data_in(2) xor data_in(3) xor data_in(5) xor data_in(6) xor data_in(7);
-- 			lfsr_q(14) <= lfsr_q(6) xor lfsr_q(26) xor lfsr_q(27) xor lfsr_q(28) xor lfsr_q(30) xor lfsr_q(31) xor data_in(2) xor data_in(3) xor data_in(4) xor data_in(6) xor data_in(7);
-- 			lfsr_q(15) <= lfsr_q(7) xor lfsr_q(27) xor lfsr_q(28) xor lfsr_q(29) xor lfsr_q(31) xor data_in(3) xor data_in(4) xor data_in(5) xor data_in(7);
-- 			lfsr_q(16) <= lfsr_q(8) xor lfsr_q(24) xor lfsr_q(28) xor lfsr_q(29) xor data_in(0) xor data_in(4) xor data_in(5);
-- 			lfsr_q(17) <= lfsr_q(9) xor lfsr_q(25) xor lfsr_q(29) xor lfsr_q(30) xor data_in(1) xor data_in(5) xor data_in(6);
-- 			lfsr_q(18) <= lfsr_q(10) xor lfsr_q(26) xor lfsr_q(30) xor lfsr_q(31) xor data_in(2) xor data_in(6) xor data_in(7);
-- 			lfsr_q(19) <= lfsr_q(11) xor lfsr_q(27) xor lfsr_q(31) xor data_in(3) xor data_in(7);
-- 			lfsr_q(20) <= lfsr_q(12) xor lfsr_q(28) xor data_in(4);
-- 			lfsr_q(21) <= lfsr_q(13) xor lfsr_q(29) xor data_in(5);
-- 			lfsr_q(22) <= lfsr_q(14) xor lfsr_q(24) xor data_in(0);
-- 			lfsr_q(23) <= lfsr_q(15) xor lfsr_q(24) xor lfsr_q(25) xor lfsr_q(30) xor data_in(0) xor data_in(1) xor data_in(6);
-- 			lfsr_q(24) <= lfsr_q(16) xor lfsr_q(25) xor lfsr_q(26) xor lfsr_q(31) xor data_in(1) xor data_in(2) xor data_in(7);
-- 			lfsr_q(25) <= lfsr_q(17) xor lfsr_q(26) xor lfsr_q(27) xor data_in(2) xor data_in(3);
-- 			lfsr_q(26) <= lfsr_q(18) xor lfsr_q(24) xor lfsr_q(27) xor lfsr_q(28) xor lfsr_q(30) xor data_in(0) xor data_in(3) xor data_in(4) xor data_in(6);
-- 			lfsr_q(27) <= lfsr_q(19) xor lfsr_q(25) xor lfsr_q(28) xor lfsr_q(29) xor lfsr_q(31) xor data_in(1) xor data_in(4) xor data_in(5) xor data_in(7);
-- 			lfsr_q(28) <= lfsr_q(20) xor lfsr_q(26) xor lfsr_q(29) xor lfsr_q(30) xor data_in(2) xor data_in(5) xor data_in(6);
-- 			lfsr_q(29) <= lfsr_q(21) xor lfsr_q(27) xor lfsr_q(30) xor lfsr_q(31) xor data_in(3) xor data_in(6) xor data_in(7);
-- 			lfsr_q(30) <= lfsr_q(22) xor lfsr_q(28) xor lfsr_q(31) xor data_in(4) xor data_in(7);
-- 			lfsr_q(31) <= lfsr_q(23) xor lfsr_q(29) xor data_in(5);
		end if;
	end if;
end process;

-------------------------------------------------------------------------------
-- CRC read
-------------------------------------------------------------------------------
process(rstn,clk)
begin
	if rstn='0' then
		crc_word<=(others=>'0');
		data_en_1ck<='0';
		crc_is_good<='0';

	elsif rising_edge(clk) then
		data_en_1ck<=data_en;
		if (data_en_1ck='0') then
			if (is_check) then
				if (crc_word=X"21_44_DF_1C") then
					crc_is_good<='1';
				else
					crc_is_good<='0';
				end if;
			else
				crc_word<=X"00" & crc_word(31 downto 8);
			end if;
		else
			crc_is_good<='0';
			for i in 0 to 31 loop
				crc_word(i)<=not(lfsr_q(31-i));
			end loop;
		end if;
	end if;
end process;
checksum<=crc_word(7 downto 0);
-- crc_word_bis<=not(crc_word);
-- lbl_revert : for i in 0 to 31 generate
-- 	crc_word_ter(i)<= not(crc_word(31-i));
-- end generate lbl_revert;
end RTL;
	
