---------------------------------------------------------------------------------------------------
-- FRAME 0 : normal, exact min length
-- FRAME 1 : 0x8000 type frame (undefined, but not a length), min length +1
-- FRAME 2 : min length with error 
-- FRAME 3 : frame requiring large padding
-- FRAME 4 : frame requiring one padding
-- FRAME 5 : frame requiring two  padding
-- O. Bourrion 12/2013
---------------------------------------------------------------------------------------------------

library std;
use std.textio.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_textio.all;

entity tb_tri_mode_eth_mac is
end tb_tri_mode_eth_mac;

architecture simu of  tb_tri_mode_eth_mac is 
component tri_mode_eth_mac
port (
	glbl_rstn : in std_logic;
	rx_axi_rstn : in std_logic;
	tx_axi_rstn : in std_logic;
	rx_axi_clk : in std_logic;
	rx_reset_out : out std_logic;
	rx_axis_mac_tdata : out std_logic_vector(7 downto 0);
	rx_axis_mac_tvalid : out std_logic;
	rx_axis_mac_tlast : out std_logic;
	rx_axis_mac_tuser : out std_logic;
	rx_statistics_vector : out std_logic_vector(27 downto 0);
	rx_statistics_valid : out std_logic;
	tx_axi_clk : in std_logic;
	tx_reset_out : out std_logic;
	tx_axis_mac_tdata : in std_logic_vector(7 downto 0);
	tx_axis_mac_tvalid : in std_logic;
	tx_axis_mac_tlast : in std_logic;
	tx_axis_mac_tuser : in std_logic;
	tx_axis_mac_tready : out std_logic;
	tx_ifg_delay : in std_logic_vector(7 downto 0);--! Intrame gap delay extension-> unused
	tx_statistics_vector : out std_logic_vector(31 downto 0);
	tx_statistics_valid : out std_logic;
	pause_req : in std_logic;
	pause_val : in std_logic_vector(15 downto 0);
	speed_is_100 : out std_logic;
	speed_is_10_100 : out std_logic;
	gmii_txd : out std_logic_vector(7 downto 0);
	gmii_tx_en : out std_logic;
	gmii_tx_er : out std_logic;
	gmii_rxd : in std_logic_vector(7 downto 0);
	gmii_rx_dv : in std_logic;
	gmii_rx_er : in std_logic;
	rx_mac_config_vector : in std_logic_vector(79 downto 0);
	tx_mac_config_vector : in std_logic_vector(79 downto 0)
	);
end component;
------------------------------------------------------------------------------
-- types to support frame data
------------------------------------------------------------------------------
-- Tx Data and Data_valid record
type data_typ is record
  data : bit_vector(7 downto 0);        -- data
  valid : bit;                          -- data_valid
  xmit : bit;                          -- data_valid
  error : bit;                          -- data_error
end record;
type frame_of_data_typ is array (natural range <>) of data_typ;

-- Tx Data, Data_valid and underrun record
type tri_mode_eth_mac_v5_3_frame_typ is record
  columns   : frame_of_data_typ(0 to 65);-- data field
  bad_frame : boolean;                   -- does this frame contain an error?
end record;
type frame_typ_ary is array (natural range <>) of tri_mode_eth_mac_v5_3_frame_typ;

constant TB_MODE                  : string := "DEMO";

------------------------------------------------------------------------------
-- Stimulus - Frame data
------------------------------------------------------------------------------
-- The following constant holds the stimulus for the testbench. It is
-- an ordered array of frames, with frame 0 the first to be injected
-- into the core transmit interface by the testbench.
------------------------------------------------------------------------------
constant frame_data : frame_typ_ary := (
-------------
-- Frame 0
-------------
0          => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"12", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"13", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"14", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"16", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"2E", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 46
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   18      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"07", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"0A", VALID => '1', xmit => '1', ERROR => '0'),
   24      => ( DATA => X"0B", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"0C", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"0D", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"0E", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"0F", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"11", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"12", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"13", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"14", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"16", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"17", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"18", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"19", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"1A", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"1B", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"1C", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"1D", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"1E", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"1F", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"21", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"22", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"23", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"24", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"25", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"26", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"27", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"28", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"29", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"2A", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"2B", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"2C", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"2D", VALID => '1', xmit => '1', ERROR => '0'),
   59      => ( DATA => X"2E", VALID => '1', xmit => '1', ERROR => '0'), -- 46th Byte of Data
--    60      => ( DATA => X"14", VALID => '1', xmit => '1', ERROR => '0'), -- FCS1
--    61      => ( DATA => X"19", VALID => '1', xmit => '1', ERROR => '0'), -- FCS2
--    62      => ( DATA => X"D1", VALID => '1', xmit => '1', ERROR => '0'), -- FCS3
--    63      => ( DATA => X"DD", VALID => '1', xmit => '1', ERROR => '0'), -- FCS4
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false),


-------------
-- Frame 1
-------------
1          => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"80", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Type = 8000
   13      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
--    12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Type = normal
--    13      => ( DATA => X"2F", VALID => '1', xmit => '1', ERROR => '0'),
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   18      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"07", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"08", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"09", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"0A", VALID => '1', xmit => '1', ERROR => '0'),
   24      => ( DATA => X"0B", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"0C", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"0D", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"0E", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"0F", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"10", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"11", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"12", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"13", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"14", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"15", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"16", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"17", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"18", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"19", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"1A", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"1B", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"1C", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"1D", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"1E", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"1F", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"20", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"21", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"22", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"23", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"24", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"25", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"26", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"27", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"28", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"29", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"2A", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"2B", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"2C", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"2D", VALID => '1', xmit => '1', ERROR => '0'),
   59      => ( DATA => X"2E", VALID => '1', xmit => '1', ERROR => '0'),
   60      => ( DATA => X"2F", VALID => '1', xmit => '1', ERROR => '0'), -- 47th Data byte
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false),


-------------
-- Frame 2
-------------
2          => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
-- 	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	7      => ( DATA => X"AA", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"2E", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 46
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'),
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   18      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"AA", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '1'), -- Error asserted
   24      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   59      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
--    60      => ( DATA => X"A1", VALID => '1', xmit => '1', ERROR => '0'), -- FCS1
--    61      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'), -- FCS2
--    62      => ( DATA => X"BB", VALID => '1', xmit => '1', ERROR => '0'), -- FCS3
--    63      => ( DATA => X"A2", VALID => '1', xmit => '1', ERROR => '0'), -- FCS4
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

   -- Error this frame
--   bad_frame => true),
  bad_frame => false),


-------------
-- Frame 3
-------------
3          => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 03
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'), -- Therefore padding is required
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'), 
   18      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),-- Padding starts here
   19      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   20      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   21      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   22      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   23      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   24      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   25      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   26      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   27      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   28      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   29      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   30      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   31      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   32      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   33      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   34      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   35      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   36      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   37      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   38      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   39      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   40      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   41      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   42      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   43      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   44      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   45      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   46      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   47      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   48      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   49      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   50      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   51      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   52      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   53      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   54      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   55      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   56      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   57      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   58      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   59      => ( DATA => X"00", VALID => '1', xmit => '0', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false),

-------------
-- Frame 4
-------------
4          => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"2D", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 45
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'), -- Therefore padding is required
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'), 
   18      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   24      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"B0", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"AA", VALID => '1', xmit => '1', ERROR => '0'),
   59      => ( DATA => X"AA", VALID => '1', xmit => '0', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false),


-- -------------
-- -- Frame 5
-- -------------
5         => (
  columns  => (
	0      => ( DATA => X"DA", VALID => '1', xmit => '1', ERROR => '0'), -- Destination Address (DA)
	1      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	2      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	3      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
	4      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
	5      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
	6      => ( DATA => X"5A", VALID => '1', xmit => '1', ERROR => '0'), -- Source Address (5A)
	7      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
	8      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
	9      => ( DATA => X"04", VALID => '1', xmit => '1', ERROR => '0'),
   10      => ( DATA => X"05", VALID => '1', xmit => '1', ERROR => '0'),
   11      => ( DATA => X"06", VALID => '1', xmit => '1', ERROR => '0'),
   12      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   13      => ( DATA => X"2C", VALID => '1', xmit => '1', ERROR => '0'), -- Length/Type = Length = 44
   14      => ( DATA => X"01", VALID => '1', xmit => '1', ERROR => '0'), -- Therefore padding is required
   15      => ( DATA => X"02", VALID => '1', xmit => '1', ERROR => '0'),
   16      => ( DATA => X"03", VALID => '1', xmit => '1', ERROR => '0'),
   17      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'), 
   18      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   19      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   20      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   21      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   22      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   23      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   24      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   25      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   26      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   27      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   28      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   29      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   30      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   31      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   32      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   33      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   34      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   35      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   36      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   37      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   38      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   39      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   40      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   41      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   42      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   43      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   44      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   45      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   46      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   47      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   48      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   49      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   50      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   51      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   52      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   53      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   54      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   55      => ( DATA => X"00", VALID => '1', xmit => '1', ERROR => '0'),
   56      => ( DATA => X"CB", VALID => '1', xmit => '1', ERROR => '0'),
   57      => ( DATA => X"AB", VALID => '1', xmit => '1', ERROR => '0'),
   58      => ( DATA => X"AB", VALID => '1', xmit => '0', ERROR => '0'),
   59      => ( DATA => X"AB", VALID => '1', xmit => '0', ERROR => '0'),
   others  => ( DATA => X"00", VALID => '0', xmit => '0', ERROR => '0')),

  -- No error in this frame
  bad_frame => false));


------------------------------------------------------------------------------
-- CRC engine
------------------------------------------------------------------------------
function calc_crc (data : in std_logic_vector;
				 fcs  : in std_logic_vector)
return std_logic_vector is

variable crc          : std_logic_vector(31 downto 0);
variable crc_feedback : std_logic;
begin

crc := not fcs;

for I in 0 to 7 loop
  crc_feedback      := crc(0) xor data(I);

  crc(4 downto 0)   := crc(5 downto 1);
  crc(5)            := crc(6)  xor crc_feedback;
  crc(7 downto 6)   := crc(8 downto 7);
  crc(8)            := crc(9)  xor crc_feedback;
  crc(9)            := crc(10) xor crc_feedback;
  crc(14 downto 10) := crc(15 downto 11);
  crc(15)           := crc(16) xor crc_feedback;
  crc(18 downto 16) := crc(19 downto 17);
  crc(19)           := crc(20) xor crc_feedback;
  crc(20)           := crc(21) xor crc_feedback;
  crc(21)           := crc(22) xor crc_feedback;
  crc(22)           := crc(23);
  crc(23)           := crc(24) xor crc_feedback;
  crc(24)           := crc(25) xor crc_feedback;
  crc(25)           := crc(26);
  crc(26)           := crc(27) xor crc_feedback;
  crc(27)           := crc(28) xor crc_feedback;
  crc(28)           := crc(29);
  crc(29)           := crc(30) xor crc_feedback;
  crc(30)           := crc(31) xor crc_feedback;
  crc(31)           :=             crc_feedback;
end loop;

-- return the CRC result
return not crc;
end calc_crc;

signal rstn : std_logic;
signal gtx_clk : std_logic;
  
-- alias gmii_tx_clk : std_logic is gtx_clk;
signal  gmii_tx_clk : std_logic;
signal gmii_tx_en           : std_logic;
signal gmii_tx_er           : std_logic;
signal gmii_txd             : std_logic_vector(7 downto 0) := (others => '0');
alias gmii_rx_clk          : std_logic is gtx_clk;
signal gmii_rx_dv           : std_logic := '0';
signal gmii_rx_er           : std_logic := '0';
signal gmii_rxd             : std_logic_vector(7 downto 0) := (others => '0');

-- testbench control signals
signal tx_monitor_finished_1G     : boolean := false;

signal phy_speed                  : std_logic_vector(1 downto 0) := "10";
signal mac_speed                  : std_logic_vector(1 downto 0) := "10";
signal update_speed               : std_logic := '0';


signal gen_tx_data                : std_logic;
signal check_tx_data              : std_logic;
signal config_bist                : std_logic;
signal frame_error                : std_logic;
signal error_seen                 : std_logic;
signal serial_response            : std_logic;
  
-- constant dly : time := 5.8 ns; 
constant dly : time := 0 ns; 
constant gtx_period : time := 4 ns;
signal tx_axis_mac_tdata : std_logic_vector(7 downto 0);
signal tx_axis_mac_tvalid : std_logic;
signal tx_axis_mac_tlast : std_logic;
signal tx_axis_mac_tuser : std_logic;
signal tx_axis_mac_tready : std_logic;
signal rx_mac_config_vector : std_logic_vector(79 downto 0):=(others=>'0');
signal tx_mac_config_vector : std_logic_vector(79 downto 0):=(others=>'0');

begin

lbl_MAC_DUT :  tri_mode_eth_mac port map (
	glbl_rstn=>rstn,
	rx_axi_rstn=>rstn,
	tx_axi_rstn=>rstn,
	rx_axi_clk=>gtx_clk,
	rx_reset_out=>open,
	rx_axis_mac_tdata=>open,
	rx_axis_mac_tvalid=>open,
	rx_axis_mac_tlast=>open,
	rx_axis_mac_tuser=>open,
	rx_statistics_vector=>open,
	rx_statistics_valid=>open,
	tx_axi_clk=>gtx_clk,
	tx_reset_out=>open,
	tx_axis_mac_tdata=>	tx_axis_mac_tdata,
	tx_axis_mac_tvalid=>tx_axis_mac_tvalid,
	tx_axis_mac_tlast=>tx_axis_mac_tlast,
	tx_axis_mac_tuser=>tx_axis_mac_tuser,
	tx_axis_mac_tready=>tx_axis_mac_tready,
	tx_ifg_delay=>X"00",
	tx_statistics_vector=>open,
	tx_statistics_valid=>open,
	pause_req=>'0',
	pause_val=>X"0000",
	speed_is_100=>open,
	speed_is_10_100=>open,
	gmii_txd=>gmii_txd,
	gmii_tx_en=>gmii_tx_en,
	gmii_tx_er=>gmii_tx_er,
	gmii_rxd =>gmii_rxd,
	gmii_rx_dv=>gmii_rx_dv,
	gmii_rx_er=>gmii_rx_er,
	rx_mac_config_vector=>rx_mac_config_vector,
	tx_mac_config_vector=>tx_mac_config_vector
	);

  -- drives gtx_clk at 125 MHz
  p_gtx_clk : process
  begin
    gtx_clk <= '0';
	rstn<='0';
    wait for 20 ns;

	rstn<='1';
    wait for 20 ns;
    loop
      wait for gtx_period;
      gtx_clk <= '1';
      wait for gtx_period;
      gtx_clk <= '0';
    end loop;
  end process p_gtx_clk;
-- gmii_tx_clk<=gtx_clk after 2 ns;
gmii_tx_clk<=gtx_clk;
  ------------------------------------------------------------------------------
  -- Stimulus process. This process will inject frames of data into the
  -- PHY side of the receiver.
  ------------------------------------------------------------------------------
  p_stimulus : process

    ----------------------------------------------------------
    -- Procedure to inject a frame into the receiver at 1Gb/s
    ----------------------------------------------------------
    procedure send_frame_1g (current_frame : in natural) is
      variable current_col   : natural := 0;  -- Column counter within frame
      variable fcs           : std_logic_vector(31 downto 0);
    begin

      wait until gmii_rx_clk'event and gmii_rx_clk = '1';

      -- Reset the FCS calculation
      fcs         := (others => '0');

      -- Adding the preamble field
      for j in 0 to 7 loop
        gmii_rxd   <= "01010101" after dly;
        gmii_rx_dv <= '1' after dly;
        gmii_rx_er <= '0' after dly;
        wait until gmii_rx_clk'event and gmii_rx_clk = '1';
      end loop;

      -- Adding the Start of Frame Delimiter (SFD)
      gmii_rxd   <= "11010101" after dly;
      gmii_rx_dv <= '1' after dly;
      wait until gmii_rx_clk'event and gmii_rx_clk = '1';

      current_col := 0;
      gmii_rxd     <= to_stdlogicvector(frame_data(current_frame).columns(current_col).data) after dly;
      gmii_rx_dv   <= to_stdUlogic(frame_data(current_frame).columns(current_col).valid) after dly;
      gmii_rx_er   <= to_stdUlogic(frame_data(current_frame).columns(current_col).error) after dly;
      fcs          := calc_crc(to_stdlogicvector(frame_data(current_frame).columns(current_col).data), fcs);

      wait until gmii_rx_clk'event and gmii_rx_clk = '1';

      current_col := current_col + 1;
      -- loop over columns in frame.
      while frame_data(current_frame).columns(current_col).valid /= '0' loop
        -- send one column of data
        gmii_rxd   <= to_stdlogicvector(frame_data(current_frame).columns(current_col).data) after dly;
        gmii_rx_dv <= to_stdUlogic(frame_data(current_frame).columns(current_col).valid) after dly;
        gmii_rx_er   <= to_stdUlogic(frame_data(current_frame).columns(current_col).error) after dly;
        fcs          := calc_crc(to_stdlogicvector(frame_data(current_frame).columns(current_col).data), fcs);

        current_col := current_col + 1;
        wait until gmii_rx_clk'event and gmii_rx_clk = '1';

      end loop;

      -- Send the CRC.
      for j in 0 to 3 loop
         gmii_rxd   <= fcs(((8*j)+7) downto (8*j)) after dly;
         gmii_rx_dv <= '1' after dly;
         gmii_rx_er <= '0' after dly;
        wait until gmii_rx_clk'event and gmii_rx_clk = '1';
      end loop;

        -- Clear the data lines.
        gmii_rxd   <= (others => '0') after dly;
        gmii_rx_dv <=  '0' after dly;

        -- Adding the minimum Interframe gap for a receiver (8 idles)
        for j in 0 to 7 loop
          wait until gmii_rx_clk'event and gmii_rx_clk = '1';
        end loop;

    end send_frame_1g;


  begin


    -- Send four frames through the MAC and Design Exampled
    -- at each state Ethernet speed
    --      -- frame 0 = minimum length frame
    --      -- frame 1 = type frame
    --      -- frame 2 = errored frame
    --      -- frame 3 = padded frame
    -------------------------------------------------------


    -- 1 Gb/s speed
    -------------------------------------------------------
    -- Wait for the internal resets to settle
    wait for 800 ns;

    assert false
      report "Sending four frames at 1Gb/s..." & cr
      severity note;

    for current_frame in frame_data'low to frame_data'high loop
      send_frame_1g(current_frame);
    end loop;

    -- Wait for 1G monitor process to complete.
    wait until tx_monitor_finished_1G;
    wait for 10 ns;

    -- Our work here is done
    assert false
      report "Simulation stopped"
      severity failure;
  end process p_stimulus;


  ------------------------------------------------------------------------------
  -- Monitor process. This process checks the data coming out of the
  -- transmitter to make sure that it matches that inserted into the
  -- receiver.
  ------------------------------------------------------------------------------
  p_monitor : process

    ---------------------------------------------------
    -- Procedure to check a transmitted frame at 1Gb/s
    ---------------------------------------------------
    procedure check_frame_1g (current_frame : in natural) is
      variable current_col   : natural := 0;  -- Column counter within frame
      variable fcs           : std_logic_vector(31 downto 0);
      variable frame_type    : string(1 to 4) := (others => ' ');
      variable Message    : line;
    begin

      -- Reset the FCS calculation
      fcs         := (others => '0');

        wait until rising_edge (gmii_tx_clk);
      -- Parse over the preamble field
--       while gmii_tx_en /= '1' and gmii_txd /= "01010101" loop
	  report "wait preamble" severity note;
      while gmii_txd /= "01010101" or gmii_tx_en /= '1' loop
			wait until rising_edge (gmii_tx_clk);
      end loop;

      while gmii_txd = "01010101" and gmii_tx_en = '1' loop
		wait until rising_edge (gmii_tx_clk);
      end loop;
	  report "preamble passed" severity note;

      -- Parse over the Start of Frame Delimiter (SFD)
      if (gmii_txd /= "11010101") then
        assert false report "SFD not present" & cr severity error;
      end if;
      wait until rising_edge(gmii_tx_clk);
	  report "SFD passed" severity note;

      if TB_MODE = "DEMO" then

         -- Start comparing transmitted dat to received data
         assert false
           report "Comparing Transmitted Data Frames to Received Data Frames" & cr
           severity note;

         -- frame has started, loop over columns of frame
         while ((frame_data(current_frame).columns(current_col).valid)='1') loop

             assert (gmii_tx_en = to_stdulogic(frame_data(current_frame).columns(current_col).valid))
               report "gmii_tx_en incorrect" & cr
               severity error;

             if gmii_tx_en = '1' then

--                -- The transmitted Destination Address was the Source Address of the injected frame
--                if current_col < 6 then
-- 					Write ( Message, string'("gmii_txd incorrect t1: "));
-- 					hWrite ( Message, gmii_txd);	
-- 					Write ( Message, string'(" ref_frame: "));
-- 					hWrite ( Message, to_stdlogicVector(frame_data(current_frame).columns(current_col+6).data));	
--                  assert (gmii_txd(7 downto 0) =
--                        to_stdlogicvector(frame_data(current_frame).columns(current_col+6).data(7 downto 0)))
-- 						report Message.all severity error;
-- 						DEALLOCATE(Message);
-- 
--                -- The transmitted Source Address was the Destination Address of the injected frame
--                elsif current_col >= 6 and current_col < 12 then
--                  assert (gmii_txd(7 downto 0) =
--                        to_stdlogicvector(frame_data(current_frame).columns(current_col-6).data(7 downto 0)))
--                    report "gmii_txd incorrect t2" & cr
--                    severity error;
-- 
--                -- for remainder of frame
--                else
					Write ( Message, string'("gmii_txd incorrect t3: "));
					hWrite ( Message, gmii_txd);	
					Write ( Message, string'(" ref_frame: "));
					hWrite ( Message, to_stdlogicVector(frame_data(current_frame).columns(current_col).data));	
                 assert (gmii_txd(7 downto 0) =
                       to_stdlogicvector(frame_data(current_frame).columns(current_col).data(7 downto 0)))
						report Message.all severity error;
						DEALLOCATE(Message);
--                end if;
           end if;

           -- calculate expected crc for the frame
           fcs        := calc_crc(gmii_txd, fcs);
-- 		   if (current_col<10) then
-- 				Write ( Message, string'("current fcs: "));
-- 				hWrite ( Message, fcs);	
-- 				Write ( Message, string'(" with txd: "));
-- 					hWrite ( Message, gmii_txd);	
-- 				report Message.all severity note;
-- 				DEALLOCATE(Message);
-- 			end if;

           -- wait for next column of data
           current_col        := current_col + 1;
           wait until gmii_tx_clk'event and gmii_tx_clk = '1';
         end loop;  -- while data valid

         -- Check the FCS matches that expected from calculation
         -- Having checked all data columns, txd must contain FCS.
		Write ( Message, string'(" full expected FCS: "));
		hWrite ( Message, fcs);	
		report Message.all severity note;
		DEALLOCATE(Message);
         for j in 0 to 3 loop
           assert (gmii_tx_en = '1')
             report "gmii_tx_en incorrect during FCS field"  & cr
             severity note;

			Write ( Message, string'("gmii_txd incorrect during FCS field, gtxd=: "));
			hWrite ( Message, gmii_txd);	
			Write ( Message, string'(" ref FCS: "));
			hWrite ( Message, fcs(((8*j)+7) downto (8*j)));	
			Write ( Message, string'(" full FCS: "));
			hWrite ( Message, fcs);	
           assert (gmii_txd = fcs(((8*j)+7) downto (8*j)))
			report Message.all severity error;
			DEALLOCATE(Message);

           wait until gmii_tx_clk'event and gmii_tx_clk = '1';
         end loop;  -- j

      else
         frame_type     := (others => ' ');
         while (gmii_tx_en='1') loop
           if current_col = 12 and gmii_txd = X"81" then
              frame_type := "VLAN";
           end if;
           -- wait for next column of data
           current_col        := current_col + 1;
           wait until gmii_tx_clk'event and gmii_tx_clk = '1';
         
         end loop;  -- while data valid
         assert false
           report frame_type & " Frame tramsmitted : Size " & integer'image(current_col) & cr
           severity note;
      end if;
    end check_frame_1g;


    variable f                  : tri_mode_eth_mac_v5_3_frame_typ;       -- temporary frame variable
    variable current_frame      : natural   := 0;  -- current frame pointer
	variable message : line;


  begin  -- process p_monitor


    -- Compare the transmitted frame to the received frames
    --      -- frame 0 = minimum length frame
    --      -- frame 1 = type frame
    --      -- frame 2 = errored frame
    --      -- frame 3 = padded frame
    -- Repeated for all stated speeds.
    -------------------------------------------------------

    -- wait for reset to complete before starting monitor to ignore false startup errors
    wait until rstn'event and rstn = '1';

    
       -- 1 Gb/s speed
       -------------------------------------------------------

       current_frame      := 0;
       

       -- Look for 1Gb/s frames.
       -- loop over all the frames in the stimulus record
       loop

         -- If the current frame had an error inserted then it would have been
         -- dropped by the FIFO in the design example.  Therefore move immediately
         -- on to the next frame.
         while frame_data(current_frame).bad_frame loop
           current_frame := current_frame + 1;
         if current_frame = frame_data'high + 1 then
             exit;
           end if;
         end loop;

         -- There are only 4 frames in this test.
         if current_frame = frame_data'high + 1 then
           exit;
         end if;

         -- Check the current frame
		Write ( Message, string'(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> FRAME "));
		Write ( Message, current_frame);	
		Write ( Message, string'(" <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"));
		report Message.all severity note;
		DEALLOCATE(Message);
         check_frame_1g(current_frame);

         -- move to the next frame
         if current_frame = frame_data'high then
           exit;
         else
           current_frame := current_frame + 1;
         end if;

       end loop;

		wait for 1000 ns;	   
		report " all frame passed" severity error;

       wait for 200 ns;
       tx_monitor_finished_1G <= true;
       wait;
  end process p_monitor;


-------------------------------------------------------------------------------
proc_stim_tx_mac : process
    
procedure xmit_frame_1g (current_frame : in natural) is
  variable current_col   : natural := 0;  -- Column counter within frame
  variable fcs           : std_logic_vector(31 downto 0);
  variable frame_type    : string(1 to 4) := (others => ' ');
begin
  
	tx_axis_mac_tdata<=(others=>'0');
	tx_axis_mac_tvalid<='0';
	tx_axis_mac_tlast<='0';
	tx_axis_mac_tuser<='0';
	wait until rising_edge(gtx_clk);

	 while ((frame_data(current_frame).columns(current_col).xmit)='1' and tx_axis_mac_tlast='0') loop
		 if (tx_axis_mac_tready='1') then
			 current_col:=current_col+1;
		end if;
		 
		tx_axis_mac_tdata<=to_stdlogicvector(frame_data(current_frame).columns(current_col).data);
		tx_axis_mac_tuser<=to_stdUlogic(frame_data(current_frame).columns(current_col).error);
		tx_axis_mac_tvalid<=to_stdUlogic(frame_data(current_frame).columns(current_col).xmit);

		if ((frame_data(current_frame).columns(current_col+1).xmit)='0') then
			tx_axis_mac_tlast<='1';
		end if;
		wait until rising_edge(gtx_clk);

	end loop;

	tx_axis_mac_tuser<='0';
	tx_axis_mac_tvalid<='0';
	tx_axis_mac_tlast<='0';
	tx_axis_mac_tvalid<='0';
	wait until rising_edge(gtx_clk);
	 
	while ((frame_data(current_frame).columns(current_col).valid)='1' ) loop
			 current_col:=current_col+1;
		wait until rising_edge(gtx_clk);
	end loop;

end xmit_frame_1g;
	variable f                  : tri_mode_eth_mac_v5_3_frame_typ;       -- temporary frame variable
    variable current_frame      : natural   := 0;  -- current frame pointer
	variable message : line;
begin
	tx_axis_mac_tdata<=(others=>'0');
	tx_axis_mac_tvalid<='0';
	tx_axis_mac_tlast<='0';
	tx_axis_mac_tuser<='0';
    wait until rstn'event and rstn = '1';

   current_frame      := 0;
   loop
		while frame_data(current_frame).bad_frame loop
			current_frame := current_frame + 1;
			if current_frame = frame_data'high + 1 then
				exit;
			end if;
		end loop;

		-- There are only 4 frames in this test.
		if current_frame = frame_data'high + 1 then
			exit;
		end if;

		-- Check the current frame
		Write ( Message, string'(">>>> FRAME XMIT "));
		Write ( Message, current_frame);	
		Write ( Message, string'(" <<<<<<<<<<<<< "));
		report Message.all severity note;
		DEALLOCATE(Message);
		xmit_frame_1g(current_frame);

		-- move to the next frame
		if current_frame = frame_data'high then
			exit;
		else
			current_frame := current_frame + 1;
		end if;
   end loop;

   wait for 2000 ns;
   report "TX loop DONE" severity error;
	wait;
end process;

end simu;
