-------------------------------------------------------------------------------
-- serial_unsigned_division
-- Operation is performed serially, takes dividend_size+2 clock cycles
-- for divisor =0 unexpected result
-- note that for divisor>=2 MSB of remain is necessarily=0 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.Std_Logic_1164.all;
use ieee.numeric_std.all;
 
entity serial_unsigned_division is
generic(
	dividend_size : natural :=32;
	divisor_size : natural :=4
	);
port(
	clk : in  std_logic;
	reset : in  std_logic;
	load  : in  std_logic;
	dividend : in  unsigned(dividend_size-1 downto 0);
	divisor : in  unsigned(divisor_size-1 downto 0);

	ready : out std_logic;
	quotient : out unsigned(dividend_size-1 downto 0);
	remain : out unsigned(dividend_size-1 downto 0)
	);
end entity serial_unsigned_division;
 
architecture RTL of serial_unsigned_division is
-- type def_state_div is (idle,compute_1,compute_2);
type def_state_div is (idle,compute_2);
signal state_div : def_state_div;

signal extended_div : unsigned(2*dividend'length-1 downto 0);
signal divisor_sav : unsigned(divisor_size-1 downto 0);
signal div_in_prog : unsigned(divisor'length-1 downto 0);
signal sub : unsigned(divisor'length downto 0);
signal cnt : natural range 0 to dividend'length;
	
signal quotient_s : unsigned(quotient'length-1 downto 0);

begin

div_in_prog<=extended_div(dividend'length+divisor'length-1 downto dividend'length);

sub<=RESIZE(div_in_prog,sub'length)-RESIZE(divisor_sav,sub'length);

process(clk)
variable extended_div_var : unsigned(2*dividend'length-1 downto 0);
begin
	if rising_edge(clk) then
-- 		sub<=RESIZE(div_in_prog,sub'length)-RESIZE(divisor_sav,sub'length);

		if reset='1' then
			ready<='0';
			state_div<=idle;
		else
			case state_div is
				when idle =>
					extended_div<=RESIZE(dividend,dividend'length+dividend'length);
					divisor_sav<=divisor;
					ready<='0';
					cnt<=0;

					if load='1' then
-- 						state_div<=compute_1;
						state_div<=compute_2;
					end if;

-- 				when compute_1 =>
-- 					state_div<=compute_2;

				when compute_2 =>
					if (sub(divisor'length)='1') then --divident is smaller than divisor
						extended_div_var:=SHIFT_LEFT(extended_div,1);
						quotient_s<=quotient_s(dividend'length-2 downto 0) & '0';

					else --dividend part is greater or equal than divisor
						extended_div_var:=SHIFT_LEFT(RESIZE(sub(divisor'length-1 downto 0) & extended_div(dividend'length-1 downto 0),2*dividend'length),1);
						quotient_s<=quotient_s(dividend'length-2 downto 0) & '1';
					end if;
					extended_div<=extended_div_var;

					if (cnt/=dividend'length) then
						cnt<=cnt+1;
-- 						state_div<=compute_1;
					else
						remain<='0' & extended_div_var(2*dividend'length-1 downto dividend'length+1);
						ready<='1';
						state_div<=idle;
					end if;
			end case;
		end if;
	end if;
end process;

quotient<=quotient_s;

end RTL;
