library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity tb_ntp_addsub is 
end entity tb_ntp_addsub;

architecture simu of tb_ntp_addsub is
component ntp_addsub is 
generic(
	is_adder : boolean :=true;
	size : natural :=64 -- must be even number here
	);
port(
	clk : in std_logic;

	op_1 : in std_logic_vector(size-1 downto 0);
	op_2 : in std_logic_vector(size-1 downto 0);

	result : out std_logic_vector(size-1 downto 0)
	);
end component ntp_addsub;
signal clk : std_logic;

signal op_1 : std_logic_vector(3 downto 0);
signal op_2 : std_logic_vector(3 downto 0);

signal expected_add_result : std_logic_vector(3 downto 0);
signal expected_sub_result : std_logic_vector(3 downto 0);

begin

process
begin
	clk<='0';
	wait for 5 ns;

	clk<='1';
	wait for 5 ns;
end process;

-------------------------------------------------------------------------------
-- DUT add
-------------------------------------------------------------------------------
lbl_DUT_add : ntp_addsub generic map(
	is_adder=>true,
	size=>4
	)
port map(
	clk=>clk,

	op_1=>op_1,
	op_2=>op_2,

	result=>open
	);
-------------------------------------------------------------------------------
-- DUT sub
-------------------------------------------------------------------------------
lbl_DUT_sub : ntp_addsub generic map(
	is_adder=>false,
	size=>4
	)
port map(
	clk=>clk,

	op_1=>op_1,
	op_2=>op_2,

	result=>open
	);
-------------------------------------------------------------------------------
-- main
-------------------------------------------------------------------------------
main : process
begin
	wait until rising_edge(clk);
	wait until rising_edge(clk);
	wait until rising_edge(clk);
	wait until rising_edge(clk);
	wait until rising_edge(clk);

	op_1<="0000";
	op_2<="0000";
	wait for 1 ns;
	expected_add_result<=op_1+op_2;
	expected_sub_result<=op_1-op_2;
	wait for 100 ns;
	wait until rising_edge(clk);

	op_1<="1000"; -- 8
	op_2<="1101"; -- 13
	wait for 1 ns;
	expected_add_result<=op_1+op_2;
	expected_sub_result<=op_1-op_2;
	wait for 100 ns;
	wait until rising_edge(clk);

	op_1<="1011"; -- 11
	op_2<="0011"; -- 3
	wait for 1 ns;
	expected_add_result<=op_1+op_2;
	expected_sub_result<=op_1-op_2;
	wait for 100 ns;
	wait until rising_edge(clk);

	op_1<="0011"; -- 3
	op_2<="1011"; -- 11
	wait for 1 ns;
	expected_add_result<=op_1+op_2;
	expected_sub_result<=op_1-op_2;
	wait for 100 ns;
	wait until rising_edge(clk);

	for i in 0 to 3 loop
	op_1<="0011"+conv_std_logic_vector(i,4); -- 3+i
	op_2<="0000"+conv_std_logic_vector(i,4); -- 0+i
	wait for 1 ns;
	expected_add_result<=op_1+op_2;
	expected_sub_result<=op_1-op_2;
	wait until rising_edge(clk);
	end loop;

	wait for 100 ns;
	report "fin de simulation" severity error;
end process main;
end simu;
