library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_arith.ALL;
use IEEE.STD_LOGIC_unsigned.ALL;
use work.pack_axi.all;
use work.pack_ipv4_types.all;
use work.pack_arp_types.all;

library unisim;
use unisim.VComponents.all;

entity top_lpsc is
generic (fast_sim : boolean :=false);
port(
	sysclk_p, sysclk_n: in STD_LOGIC;
	leds: out STD_LOGIC_VECTOR(3 downto 0);
	gmii_gtx_clk, gmii_tx_en, gmii_tx_er : out STD_LOGIC;
	gmii_txd : out STD_LOGIC_VECTOR(7 downto 0);
	gmii_rx_clk, gmii_rx_dv, gmii_rx_er: in STD_LOGIC;
	gmii_rxd : in STD_LOGIC_VECTOR(7 downto 0);
	phy_rstb : out STD_LOGIC;
	dip_switch: in std_logic_vector(3 downto 0)
	);
end top_lpsc;

architecture RTL of top_lpsc is
component UDP_Complete_nomac
	 generic (
			IN_SIMULATION     : boolean :=false;
			SIMUL_DST_MAC_NUMBER: std_logic_vector(47 downto 0):=x"AAAA_BBBB_CCCC";
			CLOCK_FREQ			: integer := 125000000;							-- freq of data_in_clk -- needed to timout cntr
			ARP_TIMEOUT			: integer := 60;									-- ARP response timeout (s)
			ARP_MAX_PKT_TMO	: integer := 5;									-- # wrong nwk pkts received before set error
			MAX_ARP_ENTRIES 	: integer := 255									-- max entries in the ARP store
			);
    Port (
			-- UDP TX signals
			udp_tx_start			: in std_logic;							-- indicates req to tx UDP
			udp_txi					: in udp_tx_type;							-- UDP tx cxns
			udp_tx_result			: out std_logic_vector (1 downto 0);-- tx status (changes during transmission)
			udp_tx_data_out_ready: out std_logic;							-- indicates udp_tx is ready to take data
			-- UDP RX signals
			udp_rx_start			: out std_logic;							-- indicates receipt of udp header
			udp_rxo					: out udp_rx_type;
			-- IP RX signals
			ip_rx_hdr				: out ipv4_rx_header_type;
			-- system signals
			rx_clk					: in  STD_LOGIC;
			tx_clk					: in  STD_LOGIC;
			reset 					: in  STD_LOGIC;
			our_ip_address 		: in STD_LOGIC_VECTOR (31 downto 0);
			our_mac_address 		: in std_logic_vector (47 downto 0);
			control					: in udp_control_type;
			-- status signals
			arp_pkt_count			: out STD_LOGIC_VECTOR(7 downto 0);			-- count of arp pkts received
			ip_pkt_count			: out STD_LOGIC_VECTOR(7 downto 0);			-- number of IP pkts received for us
			-- MAC Transmitter
			mac_tx_tdata         : out  std_logic_vector(7 downto 0);	-- data byte to tx
			mac_tx_tvalid        : out  std_logic;							-- tdata is valid
			mac_tx_tready        : in std_logic;							-- mac is ready to accept data
			mac_tx_terror        : out  std_logic;							-- tells mac to produce an error
			mac_tx_tfirst        : out  std_logic;							-- indicates first byte of frame
			mac_tx_tlast         : out  std_logic;							-- indicates last byte of frame
			-- MAC Receiver
			mac_rx_tdata         : in std_logic_vector(7 downto 0);	-- data byte received
			mac_rx_tvalid        : in std_logic;							-- indicates tdata is valid
			mac_rx_tready        : out  std_logic;							-- tells mac that we are ready to take data
			mac_rx_terror        : in  std_logic;							-- tells mac produced an error
			mac_rx_tlast         : in std_logic								-- indicates last byte of the trame
			);
end component;
component tri_mode_eth_mac
port (
	glbl_rstn : in std_logic;
	rx_axi_rstn : in std_logic;
	tx_axi_rstn : in std_logic;
	rx_axi_clk : in std_logic;
	rx_reset_out : out std_logic;
	rx_axis_mac_tdata : out std_logic_vector(7 downto 0);
	rx_axis_mac_tvalid : out std_logic;
	rx_axis_mac_tlast : out std_logic;
	rx_axis_mac_tuser : out std_logic;
	rx_statistics_vector : out std_logic_vector(27 downto 0);
	rx_statistics_valid : out std_logic;
	tx_axi_clk : in std_logic;
	tx_reset_out : out std_logic;
	tx_axis_mac_tdata : in std_logic_vector(7 downto 0);
	tx_axis_mac_tvalid : in std_logic;
	tx_axis_mac_tlast : in std_logic;
	tx_axis_mac_tuser : in std_logic;
	tx_axis_mac_tready : out std_logic;
	tx_ifg_delay : in std_logic_vector(7 downto 0);--! Intrame gap delay extension-> unused
	tx_statistics_vector : out std_logic_vector(31 downto 0);
	tx_statistics_valid : out std_logic;
	pause_req : in std_logic;
	pause_val : in std_logic_vector(15 downto 0);
	speed_is_100 : out std_logic;
	speed_is_10_100 : out std_logic;
	gmii_txd : out std_logic_vector(7 downto 0);
	gmii_tx_en : out std_logic;
	gmii_tx_er : out std_logic;
	gmii_rxd : in std_logic_vector(7 downto 0);
	gmii_rx_dv : in std_logic;
	gmii_rx_er : in std_logic;
	rx_mac_config_vector : in std_logic_vector(79 downto 0);
	tx_mac_config_vector : in std_logic_vector(79 downto 0)
	);
end component;
component mac_fifo_axi4
  port (
	 m_aclk : in std_logic;
	 s_aclk : in std_logic;
	 s_aresetn : in std_logic;
	 s_axis_tvalid : in std_logic;
	 s_axis_tready : out std_logic;
	 s_axis_tdata : in std_logic_vector(7 downto 0);
	 s_axis_tlast : in std_logic;
	 s_axis_tuser : in std_logic_vector(0 downto 0);
	 m_axis_tvalid : out std_logic;
	 m_axis_tready : in std_logic;
	 m_axis_tdata : out std_logic_vector(7 downto 0);
	 m_axis_tlast : out std_logic;
	 m_axis_tuser : out std_logic_vector(0 downto 0)
  );
end component;

signal udp_txi	: udp_tx_type;
signal our_ip_address 		: STD_LOGIC_VECTOR (31 downto 0);
signal our_mac_address 		: std_logic_vector (47 downto 0);
signal control			: udp_control_type;
signal rstn : std_logic;

signal rx_mac_config_vector : std_logic_vector(79 downto 0);
signal tx_mac_config_vector : std_logic_vector(79 downto 0);

-- MAC Transmitter
signal mac_tx_tdata         :  std_logic_vector(7 downto 0);	-- data byte to tx
signal mac_tx_tvalid        :  std_logic;							-- tdata is valid
signal mac_tx_tready        : std_logic;							-- mac is ready to accept data
signal mac_tx_terror        : std_logic;
signal mac_tx_tlast         :  std_logic;							--dicates last byte of frame
-- MAC Receiver
signal mac_rx_tdata         : std_logic_vector(7 downto 0);	-- data byte received
signal mac_rx_tvalid        : std_logic;							--dicates tdata is valid
signal mac_rx_tlast         : std_logic;
signal mac_rx_terror        : std_logic_vector(0 downto 0);
	
signal rx_data_e: std_logic_vector(7 downto 0);
signal rx_valid_e, rx_last_e,rx_rst : std_logic;
signal rx_user_ef: std_logic_vector(0 downto 0);
	
--GMII interfacing	
signal clk125: std_logic;
signal rst_125: std_logic;
signal rst_ipb: std_logic;
signal ipb_clk: std_logic;
signal sys_rst: std_logic;
signal rx_clk, rx_clk_io, clk125n: std_logic;
signal txd_e, rxd_r: std_logic_vector(7 downto 0);
signal tx_en_e, tx_er_e, rx_dv_r, rx_er_r: std_logic;
signal gmii_rxd_del: std_logic_vector(7 downto 0);
signal gmii_rx_dv_del, gmii_rx_er_del: std_logic;

begin
leds<="0000";

udp_txi.hdr.dst_ip_addr<= X"c0a86401";
udp_txi.hdr.dst_port<=conv_std_logic_vector(30,16);
udp_txi.hdr.src_port<=conv_std_logic_vector(279,16);
udp_txi.hdr.data_length<=conv_std_logic_vector(200,16);
udp_txi.hdr.checksum<=X"0000"; -- a faire calculer dans UDP TX

udp_txi.data.data_out_valid<='0';
udp_txi.data.data_out_last<='0';
udp_txi.data.data_out<=X"00";
	
-- our_mac_address<= X"020ddba115" & X"0" & X"0"; -- Careful here, arbitrary addresses do not always work
our_mac_address<= X"020ddba115" & X"3" & X"0"; -- Careful here, arbitrary addresses do not always work
our_ip_address<= X"c0a864" & X"10"; -- 192.168.100.16

control.ip_controls.arp_controls.clear_cache<='0';


lbl_UDP : UDP_Complete_nomac generic map(
			IN_SIMULATION=>false,
			SIMUL_DST_MAC_NUMBER=>x"AAAA_BBBB_CCCC",

			CLOCK_FREQ		=> 125000000,
			ARP_TIMEOUT		=> 60,
			ARP_MAX_PKT_TMO	=> 5,
			MAX_ARP_ENTRIES =>255
			)
    port map(
			-- UDP TX signals
			udp_tx_start=>'0',
			udp_txi=>udp_txi,
			udp_tx_result=>open,
			udp_tx_data_out_ready=>open,
			
			-- UDP RX signals
			udp_rx_start=>open,
			udp_rxo=>open,
			
			-- IP RX signals
			ip_rx_hdr=>open,
			
			-- system signals
			rx_clk=>clk125,
			tx_clk=>clk125,
			reset=>rst_125,
			our_ip_address=>our_ip_address,
			our_mac_address=>our_mac_address,
			control=>control,

			-- status signals
			arp_pkt_count=>open,
			ip_pkt_count=>open,
			-- MAC Transmitter
			mac_tx_tdata=>mac_tx_tdata,
			mac_tx_tvalid=>mac_tx_tvalid,
			mac_tx_tready=>mac_tx_tready,
			mac_tx_terror=>mac_tx_terror,
			mac_tx_tfirst=>open,
			mac_tx_tlast=>mac_tx_tlast,
			-- MAC Receiver
			mac_rx_tdata=>mac_rx_tdata,
			mac_rx_tvalid=>mac_rx_tvalid,
			mac_rx_terror=>mac_rx_terror(0),
			mac_rx_tready=>open,
			mac_rx_tlast=>mac_rx_tlast
			);


--	DCM clock generation for internal bus, ethernet
clocks: entity work.clocks_s6_extphy generic map (fast_sim=>fast_sim)
port map(
	sysclk_p => sysclk_p,
	sysclk_n => sysclk_n,
	clko_125 => clk125,
	clko_ipb => ipb_clk,
	locked => open,
	nuke => sys_rst,
	rsto_125 => rst_125,
	rsto_ipb => rst_ipb,
	onehz => open
	);
sys_rst<='0';	
rstn <= not rst_125;

process(rx_clk)
begin
	if rising_edge(rx_clk) then
		rx_rst <= not rst_125;
	end if;
end process;

fifo: mac_fifo_axi4 port map(
	s_aclk => rx_clk,
	s_aresetn => rx_rst,
	s_axis_tvalid => rx_valid_e,
	s_axis_tready => open,
	s_axis_tdata => rx_data_e,
	s_axis_tlast => rx_last_e,
	s_axis_tuser => rx_user_ef,
	m_aclk => clk125,
	m_axis_tvalid => mac_rx_tvalid,
	m_axis_tready => '1',
	m_axis_tdata => mac_rx_tdata,
	m_axis_tlast => mac_rx_tlast,
	m_axis_tuser => mac_rx_terror
); -- Clock domain crossing FIFO


lbl_MAC:  tri_mode_eth_mac port map (
	glbl_rstn=>rstn,
	rx_axi_rstn=>rstn,
	tx_axi_rstn=>rstn,
	rx_axi_clk => rx_clk,
	rx_reset_out=>open,
	rx_axis_mac_tdata=>rx_data_e,
	rx_axis_mac_tvalid=>rx_valid_e,
	rx_axis_mac_tlast=>rx_last_e,
	rx_axis_mac_tuser=>rx_user_ef(0),
	rx_statistics_vector=>open,
	rx_statistics_valid=>open,
	tx_axi_clk => clk125,
	tx_reset_out=>open,
	tx_axis_mac_tdata=>	mac_tx_tdata,
	tx_axis_mac_tvalid=>mac_tx_tvalid,
	tx_axis_mac_tlast=>mac_tx_tlast,
	tx_axis_mac_tuser=>mac_tx_terror,
	tx_axis_mac_tready=>mac_tx_tready,
	tx_ifg_delay=>X"00",
	tx_statistics_vector=>open,
	tx_statistics_valid=>open,
	pause_req=>'0',
	pause_val=>X"0000",
	speed_is_100=>open,
	speed_is_10_100=>open,
	gmii_txd => txd_e,
	gmii_tx_en => tx_en_e,
	gmii_tx_er => tx_er_e,
	gmii_rxd =>rxd_r,
	gmii_rx_dv=>rx_dv_r,
	gmii_rx_er=>rx_er_r,
	rx_mac_config_vector=>rx_mac_config_vector,
	tx_mac_config_vector=>tx_mac_config_vector
	);
rx_mac_config_vector <= X"0000_0000_0000_0000_0812";
tx_mac_config_vector <= X"0000_0000_0000_0000_0012";

phy_rstb <= '1';

bufio0: bufio2 port map(
				i => gmii_rx_clk,
				serdesstrobe=>open,
				divclk=>open,
				ioclk => rx_clk_io);

bufg0: bufg port map(i => gmii_rx_clk,o => rx_clk);
	
iodelgen: for i in 7 downto 0 generate
	begin
		iodelay: iodelay2 generic map(
				DELAY_SRC => "IDATAIN",
				IDELAY_TYPE => "FIXED"
			)
			port map(
				idatain => gmii_rxd(i),
				dataout => gmii_rxd_del(i),
				dataout2 => open,
				cal => '0',
				ce => '0',
				clk => '0',
				inc => '0',
				ioclk0 => '0',
				ioclk1 => '0',
				odatain => '0',
				tout =>open,
				dout =>open,
				busy =>open,
				rst => '0',
				t => '1'
			); -- Delay element for phase alignment
	
end generate;
	
iodelay_dv: iodelay2 generic map(
			DELAY_SRC => "IDATAIN",
			IDELAY_TYPE => "FIXED"
		)
		port map(
			idatain => gmii_rx_dv,
			dataout => gmii_rx_dv_del,
			dataout2 => open,
			cal => '0',
			ce => '0',
			clk => '0',
			inc => '0',
			ioclk0 => '0',
			ioclk1 => '0',
			odatain => '0',
			rst => '0',
			tout =>open,
			dout =>open,
			busy =>open,
			t => '1'
		); -- Delay element on rx clock for phase alignment
		
iodelay_er: iodelay2 generic map(
			DELAY_SRC => "IDATAIN",
			IDELAY_TYPE => "FIXED"
		)
		port map(
			idatain => gmii_rx_er,
			dataout => gmii_rx_er_del,
			dataout2 => open,
			cal => '0',
			ce => '0',
			clk => '0',
			inc => '0',
			ioclk0 => '0',
			ioclk1 => '0',
			odatain => '0',
			tout =>open,
			dout =>open,
			busy =>open,
			rst => '0',
			t => '1'
		); -- Delay element for phase alignment

process(rx_clk_io) -- FFs for incoming GMII data (need to be IOB FFs)
begin
	if rising_edge(rx_clk_io) then
		rxd_r <= gmii_rxd_del;
		rx_dv_r <= gmii_rx_dv_del;
		rx_er_r <= gmii_rx_er_del;
	end if;
end process;

process(clk125) -- FFs for outgoing GMII data (need to be IOB FFs)
	begin
		if rising_edge(clk125) then
			gmii_txd <= txd_e;
			gmii_tx_en <= tx_en_e;
			gmii_tx_er <= tx_er_e;
		end if;
	end process;
	
clk125n <= not clk125;
	
oddr0: oddr2 port map(
		q => gmii_gtx_clk,
		c0 => clk125,
		c1 => clk125n,
		ce => '1',
		d0 => '0',
		d1 => '1',
		r => '0',
		s => '0'
	); -- DDR register for clock forwarding

end RTL;
